<?php
/**
 * Created by PhpStorm.
 * User: phillipesilva
 * Date: 17/12/2014
 * Time: 14:39
 */

namespace Folha\Logic\Exame;

use Folha\Entities\GpeAlteocupacao;
use Folha\Entities\GpeVinculom;
use Folha\Entities\SstExame;
use Folha\Entities\SstExamefichamed;
use Folha\Entities\SstFichamedica;
use Folha\Entities\SstFormprogexames;
use Folha\Entities\SstItemexame;
use Folha\Entities\SstItemexamefichamed;
use Folha\Entities\SstItemperfilexame;
use Folha\Entities\SstPerfilexamevinculo;
use Geral\Entities\Ocorcalendario;
use HCM\Util\DateUtil;
use Geral\Util\NovoCodigo;
use HCM\Util\FPA;
use Setforms2\Shared\Registry;

class ProgramacaoExamePeriodico
{
    public static function ajustaDatasProgramacao($criterio, $inicio, $fim, $dtCompetencia, $dtInicial, $dtFinal, $nrOrg,
                                                  $tempoAtend, $qtAtendentes, $tempoMedio)
    {
        $dadosProg = self::getDadosProgramacaoExame($criterio, $inicio, $fim, $dtCompetencia, $dtInicial, $dtFinal, $nrOrg);

        //Verifica se a pessoa se enquadra na Faixa Etária.
        foreach ($dadosProg as $key => $dadoProg) {
            $dtNascPessoa  = $dadoProg['DTNASCPESSOA'];
            $dtNascPessoa  = DateUtil::getDataDeString($dtNascPessoa, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true);
            $dtFichaMedica = $dadoProg['DTFICHAMEDICA'];
            $fxEtariaIni   = $dadoProg['QTINIFAIXAETARIA'];
            $fxEtariaFim   = $dadoProg['QTFIMFAIXAETARIA'];

            $intervalo   = $dtFichaMedica->diff($dtNascPessoa);
            $idadePessoa = $intervalo->y;

            if(is_null($fxEtariaFim) || $fxEtariaFim == '') {
                $fxEtariaFim = 9999;
            }

            if(is_null($fxEtariaIni) || $fxEtariaIni == '') {
                $fxEtariaIni = 0;
            }

            if($idadePessoa < $fxEtariaIni || $idadePessoa > $fxEtariaFim) {
                unset($dadosProg[$key]);
            }
        }

        //Ordena os registros pelas datas e pelos números dos vínculos.
        usort($dadosProg, function ($dado1, $dado2)
        {
            if(DateUtil::comparaDatas($dado1['DTFICHAMEDICA'], $dado2['DTFICHAMEDICA'], DateUtil::IGUAL)) {
                if($dado1['NRVINCULOM'] == $dado2['NRVINCULOM']) {
                    return 0;
                }

                return ($dado1['NRVINCULOM'] < $dado2['NRVINCULOM']) ? -1 : 1;
            }

            return (DateUtil::comparaDatas($dado1['DTFICHAMEDICA'], $dado2['DTFICHAMEDICA'])) ? -1 : 1;
        });

        $dtFichaNew  = $dtFichaAnt = $dtInicial;
        $tempoAtend  = DateUtil::getHoraFloatDeString($tempoAtend);
        $vrTotalDisp = $tempoAtend * $qtAtendentes;
        $tempoUsado  = $nrVinculoAnt = 0;
        $dtFinal     = DateUtil::getDataDeString($dtFinal, DateUtil::FORMATO_BRASILEIRO, true);
        $dtFichaNew  = DateUtil::getDataDeString($dtFichaNew, DateUtil::FORMATO_BRASILEIRO, true);

        foreach ($dadosProg as $key => &$dadoProg) {
            $dtFichaMedAux = $dadoProg['DTFICHAMEDICA'];

            if($tempoUsado == $vrTotalDisp && DateUtil::comparaDatas($dtFichaNew, $dtFinal, DateUtil::IGUAL)) {
                break;
            }

            $nrVinculoM = $dadoProg['NRVINCULOM'];

            $idDiaUtil = self::verificaDiaUtil($dtCompetencia, $nrOrg, $nrVinculoM, $dtFichaNew);

            while($idDiaUtil == false) {
                $dtFichaNew = DateUtil::adicionaIntervalo($dtFichaNew, 1);
                $idDiaUtil  = self::verificaDiaUtil($dtCompetencia, $nrOrg, $nrVinculoM, $dtFichaNew);
            }

            if($tempoUsado == 0) {
                $where = "NRFICHAMEDICA IN (SELECT NRFICHAMEDICA
                                              FROM SST_FICHAMEDICA FCM
                                             WHERE TO_CHAR(FCM.DTHRPREVFICHAMED, 'DD/MM/YYYY') = ?
                                               AND FCM.DTHREFETFICHAMED IS NULL
                                               AND FCM.NRVINCULOM <> ?)
                      AND NRORG = ?";

                $params = array($dtFichaNew->format('d/m/Y'), $nrVinculoM, $nrOrg);

                $fichasMedicas = SstFichamedica::getObjectsByWhere($where, $params);

                if(!empty($fichasMedicas)) {
                    $tempoUsado = (count($fichasMedicas) * $tempoMedio) / 60;
                } else {
                    $tempoUsado = 0;
                }
            }

            //Se o vínculo em questão for diferente do vínculo anterior, incrementa o tempo usado.
            if((is_null($nrVinculoAnt) || $nrVinculoAnt != $nrVinculoM) && $key != 0) {
                $tempoUsado += ($tempoMedio / 60);
            }

            if($nrVinculoM == $nrVinculoAnt) {
                $dadoProg['DTFICHAMEDICA'] = $dtFichaAnt;
            } else {

                //Verifica se foi utilizado todo o tempo disponível ou se mais um atendimento ultrapassa o tempo disponível.
                if($tempoUsado == $vrTotalDisp || $tempoUsado + ($tempoMedio / 60) > $vrTotalDisp) {
                    $dtFichaNew = DateUtil::adicionaIntervalo($dtFichaNew, 1);
                    $tempoUsado = 0;
                }

                $dadoProg['DTFICHAMEDICA'] = $dtFichaNew;
                $dtFichaAnt = $dtFichaNew;
            }

            //Verifica se para a data previamente definida existe uma ficha médica.
            if(!DateUtil::comparaDatas($dtFichaMedAux, $dadoProg['DTFICHAMEDICA'], DateUtil::DEPOIS)) {
                $where = "NRFICHAMEDICA IN (SELECT NRFICHAMEDICA
                                              FROM SST_FICHAMEDICA FCM
                                             WHERE TO_CHAR(FCM.DTHRPREVFICHAMED, 'DD/MM/YYYY') = ?
                                               AND FCM.DTHREFETFICHAMED IS NULL
                                               AND FCM.NRVINCULOM = ?)
                      AND NRORG = ?";

                $params = array($dtFichaMedAux->format('d/m/Y'), $nrVinculoM, $nrOrg);

                $fichaMedica = SstFichamedica::getObjectByWhere($where, $params);

                if(is_object($fichaMedica)) {
                    $dadoProg['DTFICHAMEDICA'] = $dtFichaMedAux;

                    if(is_null($nrVinculoAnt) || $nrVinculoAnt != $nrVinculoM) {
                        $tempoUsado -= ($tempoMedio / 60);
                    }
                }
            }

            //Verifica se a data programada está dentro das datas de vigência.
            $where = "NRPERFILEXAMEVINCULO IN (SELECT PEV.NRPERFILEXAMEVINCULO
                                                 FROM (SELECT PEV.NRPERFILEXAMEVINCULO, PEV.NRORG, PEV.NRPERFILEXAME
                                                         FROM SST_PERFILEXAMEVINCULO PEV
                                                        WHERE ? BETWEEN DTINIVIGENCIA AND NVL(DTFIMVIGENCIA, ?)
                                                          AND NRVINCULOM = ?) PEV
                                                INNER JOIN (SELECT NREXAME, NRORG, NRPERFILEXAME
                                                              FROM SST_ITEMPERFILEXAME
                                                             WHERE ? BETWEEN DTINIVIGENCIA AND NVL(DTFIMVIGENCIA, ?)) IPE
                                                   ON IPE.NRPERFILEXAME = PEV.NRPERFILEXAME
                                                  AND IPE.NRORG = PEV.NRORG
                                                WHERE IPE.NREXAME = ?
                                                  AND IPE.NRORG = ?)
                  AND NRORG = ?";

            $params = array($dadoProg['DTFICHAMEDICA'], $dadoProg['DTFICHAMEDICA'], $nrVinculoM,
                            $dadoProg['DTFICHAMEDICA'], $dadoProg['DTFICHAMEDICA'], $dadoProg['NREXAME'], $nrOrg, $nrOrg);

            $perfis = SstPerfilexamevinculo::getObjectsByWhere($where, $params);

            $verificaVigencia = count($perfis);

            if($verificaVigencia == 0) {
                unset($dadosProg[$key]);
            }

            $nrVinculoAnt = $nrVinculoM;
        }

        //Ordena novamente os registros pelas datas e pelos números dos vínculos, pois podem existir fichas já cadastradas
        //com datas anteriores às geradas durante a programação.
        usort($dadosProg, function ($dado1, $dado2)
        {
            if(DateUtil::comparaDatas($dado1['DTFICHAMEDICA'], $dado2['DTFICHAMEDICA'], DateUtil::IGUAL)) {
                if($dado1['NRVINCULOM'] == $dado2['NRVINCULOM']) {
                    return 0;
                }

                return ($dado1['NRVINCULOM'] < $dado2['NRVINCULOM']) ? -1 : 1;
            }

            return (DateUtil::comparaDatas($dado1['DTFICHAMEDICA'], $dado2['DTFICHAMEDICA'])) ? -1 : 1;
        });

        $arrayProg = array();
        reset($dadosProg);

        //Agrupa os registros pelas datas.
        if($dadosProg) {
            do {
                $a = current($dadosProg);
                $formatDtFicha = $a['DTFICHAMEDICA']->format('d/m/Y');
                $arrayProg[$formatDtFicha][] = current($dadosProg);
                next($dadosProg);
            } while (key($dadosProg) <= count($dadosProg) - 1 && !is_null(key($dadosProg)));
        }

        return $arrayProg;
    }

    private static function getDadosProgramacaoExame($criterio, $inicio, $fim, $dtCompetencia, $dtInicial, $dtFinal, $nrOrg)
    {
        $itensPerfilRep = SstItemperfilexame::getRepository();
        $itensPerfil    = $itensPerfilRep->getItensPerfilExameVinculo($criterio, $inicio, $fim, $dtCompetencia, $dtInicial);

        foreach ($itensPerfil as $key => &$item) {
            //Gera a data da programação inicial verificando se já existem fichas para o vínculo
            $dtFichaMedica = self::geraDataProgramacao($item['NRVINCULOM'], $item['QTPERIODMESES'], $dtInicial, $dtFinal, $nrOrg, $item['NREXAME']);

            //Se a data for nula, o exame não entrará na programação.
            if(!is_null($dtFichaMedica)) {
                $item['DTFICHAMEDICA'] = $dtFichaMedica;
            } else {
                unset($itensPerfil[$key]);
            }
        }

        return $itensPerfil;
    }

    private static function geraDataProgramacao($nrVinculoM, $qtPeriodoMeses, $dtInicial, $dtFinal, $nrOrg, $nrExame)
    {
        //Busca a data Ficha Médica mais recente do vínculo
        $where = "NRFICHAMEDICA = (SELECT NRFICHAMEDICA
                                     FROM SST_FICHAMEDICA FIC
                                    WHERE NRVINCULOM = ?
                                      AND NRORG = ?
                                      AND NVL(FIC.DTHREFETFICHAMED, FIC.DTHRPREVFICHAMED) IN
                                                                    (SELECT MAX(NVL(FIC.DTHREFETFICHAMED, FIC.DTHRPREVFICHAMED))
                                                                      FROM SST_FICHAMEDICA FIC
                                                                     INNER JOIN SST_EXAMEFICHAMED EXF
                                                                        ON EXF.NRFICHAMEDICA = FIC.NRFICHAMEDICA
                                                                       AND EXF.NRORG = FIC.NRORG
                                                                     WHERE FIC.NRVINCULOM = ?
                                                                       AND FIC.NRORG = ?
                                                                       AND EXF.NREXAME = ?))
              AND NRORG = ?";

        $params = array($nrVinculoM, $nrOrg, $nrVinculoM, $nrOrg, $nrExame, $nrOrg);

        $fichaMed = SstFichamedica::getObjectByWhere($where, $params);

        /** @var SstFichamedica $fichaMed */
        if(is_object($fichaMed)) {
            $dtUltFicha = $fichaMed->getDthrefetfichamed();

            if(is_null($dtUltFicha)) {
                $dtUltFicha = $fichaMed->getDthrprevfichamed();
            }
        } else {
            //Caso não exista Ficha Médica, busca a data de início mais recente da ocupação do vínculo
            $whereOcup = "NRALTEOCUPACAO = (SELECT NRALTEOCUPACAO
                                              FROM GPE_ALTEOCUPACAO AOC
                                             WHERE AOC.NRVINCULOM = ?
                                               AND AOC.NRORG = ?
                                               AND AOC.DTFIMOCUPACAO IS NULL
                                               AND TO_DATE(TO_CHAR(AOC.DTINIOCUPACAO, 'DD/MM/YYYY')) <= TO_DATE(?)
                                               AND AOC.DTINIOCUPACAO IN (SELECT MAX(AOC.DTINIOCUPACAO)
                                                                           FROM GPE_ALTEOCUPACAO AOC
                                                                          WHERE AOC.NRVINCULOM = ?
                                                                            AND AOC.NRORG = ?
                                                                            AND AOC.DTFIMOCUPACAO IS NULL))
                      AND NRORG = ?";

            $paramsOcup = array($nrVinculoM, $nrOrg, $dtInicial, $nrVinculoM, $nrOrg, $nrOrg);

            /** @var GpeAlteocupacao $alteOcupacao */
            $alteOcupacao = GpeAlteocupacao::getObjectByWhere($whereOcup, $paramsOcup);

            if(is_object($alteOcupacao)) {
                $dtUltFicha = $alteOcupacao->getDtiniocupacao();
            } else {
                $dtUltFicha = null;
            }
        }

        $dtInicial = DateUtil::getDataDeString($dtInicial, DateUtil::FORMATO_BRASILEIRO, true);
        $dtFinal   = DateUtil::getDataDeString($dtFinal,   DateUtil::FORMATO_BRASILEIRO, true);

        //Se existir data e a mesma estiver antes do período da Programação.
        if(!is_null($dtUltFicha) && DateUtil::comparaDatas($dtUltFicha, $dtInicial, DateUtil::ANTES)) {
            if (is_null($qtPeriodoMeses)) {
                $dtUltFicha = null;
            } else {
                //Adiciona o período (em meses) do exame na data da ficha até o início do período.
                while (DateUtil::comparaDatas($dtUltFicha, $dtInicial)) {
                    $dtUltFicha = DateUtil::adicionaIntervalo($dtUltFicha, $qtPeriodoMeses, DateUtil::MES);
                }

                //Se a data gerada é maior que a data final do período
                if (DateUtil::comparaDatas($dtFinal, $dtUltFicha)) {
                    $dtUltFicha = null;
                }
            }
        } else {
            $dtUltFicha = null;
        }

        return $dtUltFicha;
    }

    private static function verificaDiaUtil($dtCompetencia, $nrOrg, $nrVinculoM, $dtFichaMedica)
    {
        $where = "NROCORCALEND IN (SELECT NROCORCALEND
                                     FROM OCORCALENDARIO ORC
                                     JOIN ESTRUTURAH ETH
                                       ON (ORC.NRCALENDARIO = ETH.NRCALENDARIO
                                      AND  ORC.NRORG = ETH.NRORG)
                                     JOIN GPE_VINCULOH VIH
                                       ON (ETH.NRESTRUTURAM = VIH.NRESTRUTLEGAL)
                                    WHERE VIH.DTMESCOMPETENC IN (SELECT MAX(VIH.DTMESCOMPETENC)
                                                                   FROM GPE_VINCULOH VIH
                                                                  WHERE VIH.NRVINCULOM  = ?
                                                                    AND VIH.NRORG = ?
                                                                    AND TO_CHAR(VIH.DTMESCOMPETENC,'DD/MM/YYYY') <= '$dtCompetencia')
                                      AND VIH.NRORG = ?
                                      AND VIH.NRVINCULOM = ?
                                      AND TO_CHAR(ORC.DTCALENDARIO,'DD/MM/YYYY') = ?)
              AND NRORG = ?";

        $dtFichaMedica = $dtFichaMedica->format('d/m/Y');
        $params = array($nrVinculoM, $nrOrg, $nrOrg, $nrVinculoM, $dtFichaMedica, $nrOrg);

        $calendario = Ocorcalendario::getObjectsByWhere($where, $params);

        if(empty($calendario)) {
            return true;
        }

        return false;
    }

    public static function organizaDataSourceDadosProgPorVinculo($dadosProg)
    {
        $orgDataSource = array();

        foreach($dadosProg as $key => $dataProg) {
            do {
                $a = current($dataProg);
                $nrVinculoM = $a['NRVINCULOM'];
                $orgDataSource[$key][$nrVinculoM][] = current($dataProg);
                next($dataProg);
            } while(key($dataProg) <= count($dataProg) - 1 && !is_null(key($dataProg)));
        }

        return $orgDataSource;
    }

    public static function getDataSourceFichaMedica($dadosProg)
    {
        $dataSource = array();

        foreach ($dadosProg as $keyData => $datas) {
            foreach ($datas as $keyVinculo => $vinculos) {
                $vinculoM   = GpeVinculom::findOneByNrvinculomAndNrorg($keyVinculo, FPA::getNrorgTrabalho());
                $nmVinculoM = $vinculoM->getNmvinculom();

                $dataSource[] = array('NRVINCULOM'        => $keyVinculo,
                                      'DTHRPREVFICHAMED'  => $keyData,
                                      'NMVINCULOM'        => $nmVinculoM);
            }
        }

        if(empty($dataSource)) {
            return array(array());
        }

        return $dataSource;
    }

    public static function getDataSourceExamesFichaMedica($dadosProg, $dtProg, $nrVinculo)
    {
        $dataSource = array();
        $nrExamesCadastrados = array();

        //Verifica se já existe uma ou mais Fichas Médicas cadastradas que não foram efetivadas para o dia e adiciona
        //seus exames para serem visualizados na simulação, mas que não serão novamente gravados.
        $where = "NRVINCULOM = ?
              AND TO_CHAR(DTHRPREVFICHAMED, 'DD/MM/YYYY') = ?
              AND DTHREFETFICHAMED IS NULL
              AND NRORG = ?";

        $params = array($nrVinculo, $dtProg, FPA::getNrorgTrabalho());

        $fichasMedicas = SstFichamedica::getObjectsByWhere($where, $params);

        foreach ($fichasMedicas as $ficha) {
            $examesFicha = SstExamefichamed::findByNrfichamedicaAndNrorg($ficha->getNrfichamedica(), FPA::getNrorgTrabalho());

            foreach ($examesFicha as $exame) {
                $sstExame = SstExame::findOneByNrexameAndNrorg($exame->getNrexame(), FPA::getNrorgTrabalho());

                if(is_null($sstExame)) {
                    $sstExame = SstExame::findOneByNrexameAndNrorg($exame->getNrexame(), FPA::getNrorgPadrao());
                }

                $nomeExame = $sstExame->getNmexame();

                $dataSource[] = array('NOME'         => $nomeExame,
                                      'IDEXAME'      => $exame->getIdexamerefseq(),
                                      'EXAME'        => $exame->getNrexame(),
                                      'EXAME__LABEL' => $nomeExame);

                $nrExamesCadastrados[] = $exame->getNrexame();
            }
        }

        //Exames da Programação que não existam nas Fichas já cadastradas.
        foreach ($dadosProg[$dtProg][$nrVinculo] as $exame) {
            if(!in_array($exame['NREXAME'], $nrExamesCadastrados)) {
                $sstExame = SstExame::findOneByNrexameAndNrorg($exame['NREXAME'], FPA::getNrorgTrabalho());

                if(is_null($sstExame)) {
                    $sstExame = SstExame::findOneByNrexameAndNrorg($exame['NREXAME'], FPA::getNrorgPadrao());
                }

                $nome = $sstExame->getNmexame();

                $dataSource[] = array('NOME'         => $nome,
                                      'IDEXAME'      => 'S',
                                      'EXAME'        => $exame['NREXAME'],
                                      'EXAME__LABEL' => $nome);
            }
        }

        if(empty($dataSource)) {
            return array(array());
        }

        return $dataSource;
    }

    public static function getDataSourceItensExameFichaMedica($nrExame)
    {
        $dataSource = array();
        $where = "NREXAME = ?
              AND (NRORG = ? OR NRORG = ?)";

        $params = array($nrExame, FPA::getNrorgTrabalho(), FPA::getNrorgPadrao());

        $itensExame = SstItemexame::getObjectsByWhere($where, $params);

        foreach ($itensExame as $item) {
            $dataSource[] = array('NOME'             => $item->getNmitemexame(),
                                  'IDITEMEXAME'      => 'S',
                                  'ITEMEXAME'        => $item->getNritemexame(),
                                  'ITEMEXAME__LABEL' => $item->getNmitemexame());
        }

        if(empty($dataSource)) {
            return array(array());
        }

        return $dataSource;
    }

    public static function gravarProgramacao($dadosProg)
    {
        $entityManager = Registry::getEntityManager();
        $db = $entityManager->getConnection();
        $db->beginTransaction();

        try {
            $nrOrg      = FPA::getNrorgTrabalho();
            $dataAtual  = DateUtil::getDataAtual();
            $cdOperador = FPA::getCdOperador();

            foreach ($dadosProg as $keyData => $data) {
                foreach ($data as $keyVinculo => $vinculos) {
                    foreach ($vinculos as $exames) {
                        $validaFicha = self::verificaExisteFichaMedica($keyData, $keyVinculo, $nrOrg);
                        $examesCadastrados = array();

                        if(is_array($validaFicha)) {
                            $nrFichaMedica     = $validaFicha['NRFICHAMEDICA'];
                            $examesCadastrados = $validaFicha['EXAMES'];
                        } else {
                            $nrFichaMedica = self::insertFichaMedica($keyData, $keyVinculo, $nrOrg, $dataAtual, $cdOperador);
                        }

                        if(!in_array($exames['NREXAME'], $examesCadastrados)) {
                            self::insertExameFichaMedica($nrFichaMedica, $exames['NREXAME'], $nrOrg, $dataAtual, $cdOperador);
                        }
                    }
                }
            }

            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    private static function verificaExisteFichaMedica($dtFicha, $nrVinculo, $nrOrg)
    {
        //Verifica se existe alguma ficha médica para o vínculo no dia.
        //Se existir mais de uma, pega a mais recente.
        $where = "NRVINCULOM = ?
              AND TO_CHAR(DTHRPREVFICHAMED, 'DD/MM/YYYY') = ?
              AND DTHREFETFICHAMED IS NULL
              AND DTHRPREVFICHAMED IN (SELECT MAX(DTHRPREVFICHAMED)
                                         FROM SST_FICHAMEDICA
                                        WHERE NRVINCULOM = ?
                                          AND DTHREFETFICHAMED IS NULL
                                          AND TO_CHAR(DTHRPREVFICHAMED, 'DD/MM/YYYY') = ?)
              AND NRORG = ?";

        $params = array($nrVinculo, $dtFicha, $nrVinculo, $dtFicha, $nrOrg);

        $fichaMedica = SstFichamedica::getObjectByWhere($where, $params);

        if(is_object($fichaMedica)) {
            $examesCadastrados = array('NRFICHAMEDICA' => $fichaMedica->getNrfichamedica(),
                                       'EXAMES' => array());

            $examesFicha = SstExamefichamed::findByNrfichamedicaAndNrorg($fichaMedica->getNrfichamedica(), FPA::getNrorgTrabalho());

            foreach ($examesFicha as $exame) {
                $examesCadastrados['EXAMES'][] = $exame->getNrexame();
            }

            return $examesCadastrados;
        }

        return false;
    }

    private static function insertFichaMedica($dtFicha, $nrVinculo, $nrOrg, $dataAtual, $cdOperador)
    {
        $nrFichaMedica = NovoCodigo::geraCodigo($nrOrg, 'SST_FICHAMEDICA', 1, 10, true, false);

        $ficha = new SstFichamedica();

        $ficha->setNrfichamedica($nrFichaMedica);
        $ficha->setDthrprevfichamed(DateUtil::getDataDeString($dtFicha, DateUtil::FORMATO_BRASILEIRO, true));
        $ficha->setNrvinculom($nrVinculo);
        $ficha->setIdbeneficio('N');
        $ficha->setIdemissaocat('N');
        $ficha->setIdnexocausal('N');
        $ficha->setIdativo('N');
        $ficha->setNrorg($nrOrg);
        $ficha->setNrorgultatu($nrOrg);
        $ficha->setNrorginclusao($nrOrg);
        $ficha->setDtinclusao($dataAtual);
        $ficha->setDtultatu($dataAtual);
        $ficha->setCdoperinclusao($cdOperador);
        $ficha->setCdoperultatu($cdOperador);
        $ficha->persist();

        Registry::getEntityManager()->flush();

        return $nrFichaMedica;
    }

    private static function insertExameFichaMedica($nrFichaMedica, $nrExame, $nrOrg, $dataAtual, $cdOperador)
    {
        $nrExameFicha = NovoCodigo::geraCodigo($nrOrg, 'SST_EXAMEFICHAMED', 1, 10, true, false);

        $sstExame = SstExame::findOneByNrexameAndNrorg($nrExame, $nrOrg);

        if(is_null($sstExame)) {
            $sstExame = SstExame::findOneByNrexameAndNrorg($nrExame, FPA::getNrorgPadrao());
        }

        $nomeExame = $sstExame->getNmexame();

        $exame = new SstExamefichamed();

        $exame->setNrexamefichamed($nrExameFicha);
        $exame->setNmexamefichamed($nomeExame);
        $exame->setNrfichamedica($nrFichaMedica);
        $exame->setNrexame($nrExame);
        $exame->setIdocupacional('N');
        $exame->setIdresultnormal('S');
        $exame->setIdexamerefseq('S');
        $exame->setIdativo('S');
        $exame->setNrorg($nrOrg);
        $exame->setNrorgultatu($nrOrg);
        $exame->setNrorginclusao($nrOrg);
        $exame->setDtinclusao($dataAtual);
        $exame->setDtultatu($dataAtual);
        $exame->setCdoperinclusao($cdOperador);
        $exame->setCdoperultatu($cdOperador);
        $exame->persist();

        Registry::getEntityManager()->flush();

        self::insertItemExameFichaMedica($nrExameFicha, $nrExame, $nrOrg, $dataAtual, $cdOperador);
    }

    private static function insertItemExameFichaMedica($nrExameFicha, $nrExame, $nrOrg, $dataAtual, $cdOperador)
    {
        $where = "NREXAME = ?
              AND (NRORG = ? OR NRORG = ?)";

        $params = array($nrExame, $nrOrg, FPA::getNrorgPadrao());

        $itensExame = SstItemexame::getObjectsByWhere($where, $params);

        foreach ($itensExame as $item) {
            $nrItemExameFicha = NovoCodigo::geraCodigo($nrOrg, 'SST_ITEMEXAMEFICHAMED', 1, 10, true, false);

            $itemExame = new SstItemexamefichamed();

            $itemExame->setNritemexamefichamed($nrItemExameFicha);
            $itemExame->setNmitemexamefichamed($item->getNmitemexame());
            $itemExame->setNrexamefichamed($nrExameFicha);
            $itemExame->setNritemexame($item->getNritemexame());
            $itemExame->setIdocupacional('N');
            $itemExame->setIdresultnormal('S');
            $itemExame->setIdexamerefseq('S');
            $itemExame->setIdativo('S');
            $itemExame->setNrorg($nrOrg);
            $itemExame->setNrorgultatu($nrOrg);
            $itemExame->setNrorginclusao($nrOrg);
            $itemExame->setDtinclusao($dataAtual);
            $itemExame->setDtultatu($dataAtual);
            $itemExame->setCdoperinclusao($cdOperador);
            $itemExame->setCdoperultatu($cdOperador);
            $itemExame->persist();
        }

        Registry::getEntityManager()->flush();
    }

    public static function validaProgramacaoExame($nrFormProgExames, $nrOrg)
    {
        $formProgExames = SstFormprogexames::findOneByNrformprogexamesAndNrorg($nrFormProgExames, $nrOrg);

        if(is_object($formProgExames)) {
            return true;
        }

        return false;
    }
}